package com.dsk.flink.dsc.common.function;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.streaming.api.functions.ProcessFunction;
import org.apache.flink.util.Collector;

import java.text.SimpleDateFormat;
import java.util.*;
/**
 * @author lww
 * @date 2025-01-14
 */
public class MysqlDataTransferFunction extends ProcessFunction<JSONObject, Tuple3<String,String,Long>> {

    private static final Map<String,Integer> STR_SQL_TYPE;

    static {
        STR_SQL_TYPE = MapUtil.newHashMap();
        STR_SQL_TYPE.put("VARCHAR",1);
        STR_SQL_TYPE.put("CHAR",1);
        STR_SQL_TYPE.put("TINYBLOB",1);
        STR_SQL_TYPE.put("BLOB",1);
        STR_SQL_TYPE.put("MEDIUMBLOB",1);
        STR_SQL_TYPE.put("LONGBLOB",1);
        STR_SQL_TYPE.put("TINYTEXT",1);
        STR_SQL_TYPE.put("TEXT",1);
        STR_SQL_TYPE.put("MEDIUMTEXT",1);
        STR_SQL_TYPE.put("LONGTEXT",1);
        STR_SQL_TYPE.put("TIME",1);
        STR_SQL_TYPE.put("TIMESTAMP",1);
        STR_SQL_TYPE.put("JSON",1);
    }

    /*private static String tranferInsertSql(String table, JSONObject dataObj, JSONObject mysqlType) {

        Set<String> columnSet = mysqlType.keySet();
        StringBuilder sb = new StringBuilder();
        for (String s : columnSet) {
            sb.append("`").append(s).append("`,");
        }
        List<String> valueList = new ArrayList<>();
        for (String col : columnSet) {
            valueList.add(getValueString(dataObj,col,mysqlType.getString(col)));
        }
        sb.setLength(sb.length()-1);
        String columnString = sb.toString();
        String valueString = String.join(",",valueList);

        //return String.format("INSERT INTO %s (%s) values (%s) ON DUPLICATE KEY UPDATE %s;",table,columnString,valueString,updateString);
        return String.format("REPLACE INTO %s (%s) values (%s);",table,columnString,valueString);
    }*/
    private static String tranferInsertSql(String table, JSONObject dataObj, JSONObject mysqlType) {
        Set<String> columnSet = mysqlType.keySet();
        StringBuilder sb = new StringBuilder("REPLACE INTO ").append(table).append(" (");
        List<String> valueList = new ArrayList<>();
        for (String s : columnSet) {
            sb.append("`").append(s).append("`,");
            valueList.add(getValueString(dataObj, s, mysqlType.getString(s)));
        }
        sb.setLength(sb.length() - 1);
        sb.append(") values (");
        sb.append(String.join(",", valueList));
        sb.append(")");
        return sb.toString();
    }


    /*private static String transferDeleteSql(String table, JSONObject dataObj, JSONObject mysqlType, Set<String> pkNameSet) {

        List<String> whereList = new ArrayList<>();
        for (String pk : pkNameSet) {
            String whereString = pk.concat(" = ").concat(getValueString(dataObj,pk,mysqlType.getString(pk)));
            whereList.add(whereString);
        }
        String whereString = String.join(" and ",whereList);
        return String.format("DELETE FROM %s WHERE %s",table,whereString);
    }*/

    private static String transferDeleteSql(String table, JSONObject dataObj, JSONObject mysqlType, Set<String> pkNameSet) {
        StringBuilder whereClauseBuilder = new StringBuilder();
        for (String pk : pkNameSet) {
            if (whereClauseBuilder.length() > 0) {
                whereClauseBuilder.append(" and ");
            }
            whereClauseBuilder.append(pk).append(" = ").append(getValueString(dataObj, pk, mysqlType.getString(pk)));
        }
        return String.format("DELETE FROM %s WHERE %s",table, whereClauseBuilder);
    }

    /**
     * @author shezaixing
     * @date 2023/12/7 14:23
     * @description 判断拼接字符串时类型（是否需要加上引号）
     *
     */
    private static String getValueString(JSONObject dataObj,String columnKey,String mysqlType){

        if(null == dataObj.get(columnKey)){
            return "null";
        }
        String upperCase = mysqlType.toUpperCase();
        //需要处理成字符串加引号的类型
        if(STR_SQL_TYPE.containsKey(upperCase)){
            return String.format("'%s'", dataObj.getString(columnKey).replace("\\","\\\\").replace("'", "\\'") );
        }

        //时间字段处理
        if("DATE".equals(upperCase) || "DATETIME".equals(upperCase)){
            String date = "DATETIME".equals(upperCase) ? DateUtil.format(dataObj.getDate(columnKey),"yyyy-MM-dd HH:mm:ss") : DateUtil.format(dataObj.getDate(columnKey),"yyyy-MM-dd");
            return String.format("\"%s\"",date);
        }

        return dataObj.getString(columnKey);
    }

    @Override
    public void processElement(JSONObject value, Context ctx, Collector<Tuple3<String, String, Long>> out) throws Exception {
        //返回数据集合
        String type = value.getString("type");
        //JSONArray dataList = value.getJSONArray("data");
        JSONObject mysqlType = value.getJSONObject("mysqlType");
        String table = value.getString("table");
        JSONArray pkNames = value.getJSONArray("pkNames");
        Set<String> pkNameSet = new HashSet<>();
        long ts = value.getLong("ts") == null ? System.currentTimeMillis() : value.getLong("ts");
        if(CollUtil.isNotEmpty(pkNames)){
            pkNames.forEach(name -> pkNameSet.add(String.valueOf(name)));
        }
        String excueteSql;
        JSONObject dataObj = value.getJSONArray("data").getJSONObject(0);

        /*Boolean logicalDelete = MapUtil.getBool(dbInfoMap, "logical_delete", false);
        if(logicalDelete){
            mysqlType.put("is_del", "int");
            dataObj.put("is_del", "DELETE".equals(type) ? 1 : 0);
        }*/

        //处理先后顺序
        //获取该条数据的表名和主键作为唯一的groupKey
        StringBuilder groupKeyBuilder = new StringBuilder(table);
        for (String pk : pkNameSet) {
            String pkValue = getValueString(dataObj, pk, mysqlType.getString(pk));
            groupKeyBuilder.append("-").append(pkValue);
        }
        String groupKey = groupKeyBuilder.toString();

        if("INSERT".equals(type) || "UPDATE".equals(type)){
            excueteSql = tranferInsertSql(table,dataObj,mysqlType);
        } else {
            excueteSql = transferDeleteSql(table,dataObj,mysqlType,pkNameSet);
        }
        out.collect(Tuple3.of(excueteSql,groupKey,ts));
        /*if (MapUtil.getBool(dbInfoMap, "log_enable", false)){
            String logSql = buildLogData(type, table, pkNameSet, dataObj, ts, value.toJSONString());
            ctx.output(toSlideTag,Tuple3.of(logSql,"dsc_cdc_log",ts));
        }*/
    }
}
